@@ -12,11 +12,13 @@ c
 
 c             init sepics, epics
        call sopen
+
 c             generate showers until required # or
 c             time lack
 c          *** until loop*** 
        do while (.true.)
           call s1set(icon)
+
           if         (icon .ne. 0)
      *                       goto 10
        enddo
@@ -73,37 +75,38 @@ c                   init epics
      *     'If not, make "Trace f" in epicsfile', 1)
         endif
 c              incident energy sampling. initializaiton 
-        if( PrimaryFile(1:1) .eq. "+") then
-           Inp1ry = 1
-           call siniPrim(PrimaryFile)
-        else
-c           PrimaryFile may contain: e.g     ../Input/+primary 
-           Inp1ry = 0
-           i = 1
-           n = klena(PrimaryFile)
-           do i = n-1, 1, -1
-              if(PrimaryFile(i:i) .eq.  '/') then
-c                   the last / found
-                 if(PrimaryFile(i:i+1) .eq. '/+') then
-c                         it shouyld be file name with + at head
-                    Inp1ry = 1
-                    call siniPrim(PrimaryFile)
-                 endif
-                 goto 99
-              endif
-           enddo
+c           check primary and outprimary
+        if( trim(PrimaryFile) == trim(OutPrimaryFile) ) then
+           if( Light == 21 ) then
+              write(0, *)  ' PrimaryFile=OutPrimaryFile when'  
+              write(0, *) ' Light=', Light
+              stop
+           endif
         endif
- 99     continue
+c           PrimaryFile may contain: e.g     ../Input/+primary 
+c            or /-filename.   
+        call epSeePMfile(PrimaryFile, Inp1ry)
+
         if(Inp1ry .eq. 0 ) then
+           !  primary is ordinary one
            Inpxyz = 0
            Inpdir = 0
            Inperg  = 0
            Inptime = 0
            Inpdisk = 0
+           Inpwgt = 0
+           Inpwl = 0
+           Inppol = 0
+           Inpmass =0
+           Inpcn = 0
            Inpuser = 0
+!           InpLight = 0
            call ciniSPrim(PrimaryFile)
+        else
+           ! primary  is in +xxx or -xxx file
+           call epLightIOreadIni
         endif
-c
+
 c            inquire form
         call epqfrm(formx)
         if(formx .eq. 'cyl') then
@@ -148,71 +151,8 @@ c         ---------------- Error -----------------
           write(msg,*) ' no control parameters'
           call cerrorMsg(msg, 0)
        end
-       subroutine  siniPrim(file)
-       implicit none
-#include  "ZsepManager.h"
-       character(*) file
-       character*50  inpcond
-       integer klena,icon, loc
+      
 
-
-       open(Ioprim, file=file(1:klena(file) ))
-       read(Ioprim, '(a)')  inpcond
-       if( index(inpcond, "mul") .gt. 0 ) then
-          Inpmul = 1
-       else
-          Inpmul = 0
-       endif
-       if( index(inpcond, "xyz") .gt. 0  ) then
-          Inpxyz = 1
-       else
-          Inpxyz = 0
-       endif
-       if( index(inpcond, "dir")  .gt. 0 ) then
-          Inpdir =  1
-       else
-          Inpdir = 0
-       endif
-       if( index(inpcond, "sub") .gt. 0 ) then
-          Inpsubcode = 1
-       else
-          Inpsubcode = 0
-       endif
-       if( index(inpcond, "KE") .gt.  0  .or.
-     *     index(inpcond, "ke") .gt.  0   ) then
-          Inperg = 1
-       elseif( index(inpcond, "TE") .gt. 0 .or.
-     *     index(inpcond, "te") .gt. 0  ) then
-          Inperg = 2
-       else
-          Inperg = 0
-       endif
-       if(index(inpcond, "time") .gt. 0 ) then
-          Inptime = 1
-       else
-          Inptime = 0
-       endif
-
-       loc = index(inpcond, "user")
-       if( loc .gt. 0) then
-          Inpuser = 1
-       else
-          Inpuser = 0
-       endif
-
-       loc = index(inpcond, "disk")
-       if( loc .gt. 0) then
-          read(inpcond(loc+4:loc+5), *) Inpdisk
-       else
-          Inpdisk = 0
-       endif
-
-       call cskipComment(Ioprim, icon)
-       if(icon .ne. 0) then
-          call cerrorMsg("Primary file has no #----- line", 0)
-       endif
-
-       end
        subroutine ssetip3( aTrack, icon)
        implicit none
 #include  "ZsepManager.h"
@@ -226,81 +166,122 @@ c                      -1--> EOF
        integer code, subcode, chg 
        real*8 erg, xin, yin, zin, wx, wy, wz, u, norm
        integer maxfn
-       parameter (maxfn = 13)
-       character*120 buf
+       integer::totalcn
+       parameter (maxfn = 15)
        character*20 field(maxfn)
 
  10    continue
        buf = ' '
        subcode = 0
-       read(Ioprim, '(a)', end= 100 )  buf
-       call kgetField(buf, field, maxfn, nf)
 
-       if(nf .eq. 0 )  then
-          if( Inpmul .eq. 1) then
-             icon = 1
-             return             !****************
-          else
-c            neglect blank line          
-             goto 10
+      if(Inp1ry < 0 ) then
+c     read(Ioprim,  end= 100 )  buf
+         read(Ioprim,  end = 100)  code,
+     *        subcode, chg, erg,  xin, yin, zin,
+     *        wx, wy, wz, 
+     *        aTrack.wl, aTrack.p.mass,  aTrack.cn
+         if( code == -1000 ) then
+                ! read dE of each comp.
+            call epLightIOreaddE(totalcn,0)  ! 0 no need to alloc
+            icon = 1
+            return              !************************
+         endif
+         aTrack.p.code = code
+      else
+          read(Ioprim, '(a)', end= 100 )  buf
+          call kgetField(buf, field, maxfn, nf)
+          if(nf .eq. 0 )  then
+             if( Inpmul .eq. 1) then
+c                ! read dE   ; head trip ?? not needed
+c                call epLightIOreaddE(totalcn, 0)
+                icon = 1
+                return          !****************
+             else
+c     neglect blank line          
+                goto 10
+             endif
           endif
-       endif
           
-       fc = 1
-       read(field(fc), *) code
-       if(Inpsubcode .eq. 1) then
+          fc = 1
+          read(field(fc), *) code
+          if(Inpsubcode .eq. 1) then
+             fc = fc + 1
+             read(field(fc), *)  subcode
+          endif
           fc = fc + 1
-          read(field(fc), *)  subcode
-       endif
-       fc = fc + 1
-       read(field(fc), *)  chg
-       if(Inperg .gt. 0) then
-          fc = fc + 1 
-          read(field(fc), *)   erg
-       endif
-       if( Inpxyz .gt. 0 ) then
-          fc = fc + 1
-          read( field(fc), *)   xin
-          fc = fc + 1
-          read( field(fc), *)   yin
-          fc = fc + 1
-          read( field(fc), *)   zin
-       endif
+          read(field(fc), *)  chg
+          if(Inperg .gt. 0) then
+             fc = fc + 1 
+             read(field(fc), *)   erg
+          endif
+          if( Inpxyz .gt. 0 ) then
+             fc = fc + 1
+             read( field(fc), *)   xin
+             fc = fc + 1
+             read( field(fc), *)   yin
+             fc = fc + 1
+             read( field(fc), *)   zin
+          endif
 
-       if( Inpdir .gt. 0 ) then
-          fc = fc + 1
-          read( field(fc), *)   wx
-          fc = fc + 1
-          read( field(fc), *)   wy
-          fc = fc + 1
-          read( field(fc), *)   wz
+          if( Inpdir .gt. 0 ) then
+             fc = fc + 1
+             read( field(fc), *)   wx
+             fc = fc + 1
+             read( field(fc), *)   wy
+             fc = fc + 1
+             read( field(fc), *)   wz
 c              the acuracy of input data may not be enough
-          norm = sqrt( wx**2 + wy**2 + wz**2 )
-          wx = wx/norm
-          wy = wy/norm
-          wz = wz/norm
-       endif
+             norm = sqrt( wx**2 + wy**2 + wz**2 )
+             wx = wx/norm
+             wy = wy/norm
+             wz = wz/norm
+          endif
 
-       if( Inptime .gt. 0 ) then
-          fc = fc + 1
-          read( field(fc), *)   aTrack.t 
-          aTrack.t = aTrack.t*29.98d0    ! cm (t is nsec)
-       endif
+          if( Inpwgt  >  0 ) then
+             fc = fc + 1          
+             read( field(fc), *)  aTrack.wgt
+          endif
 
-       if( Inpuser .gt. 0 ) then
-          fc = fc + 1
-          read( field(fc), *)   aTrack.user
-       endif
+          if( Inpwl  >  0 ) then
+             fc = fc + 1          
+             read( field(fc), *)  aTrack.wl
+          endif
+          
+          if( Inppol  >  0 ) then
+             fc = fc + 1          
+             read( field(fc), *)  aTrack.pol
+          endif
+          
+          if(Inpmass > 0 ) then
+             fc = fc + 1
+             read( field(fc), *)  aTrack.p.mass
+          endif
 
+          if( Inpcn  >  0 ) then
+             fc = fc + 1          
+             read( field(fc), *)  aTrack.cn
+          endif
+
+          if( Inptime .gt. 0 ) then
+             fc = fc + 1
+             read( field(fc), *)   aTrack.t 
+             aTrack.t = aTrack.t*29.98d0 ! cm (t is nsec)
+          endif
+
+          if( Inpuser .gt. 0 ) then
+             fc = fc + 1
+             read( field(fc), *)   aTrack.user
+          endif
 
 c         check number of field; some redandant field may exist at the last part, so
 c         nf many not be equal to fc. but must be nf >= fc
-       if(nf .lt.  fc) then
-          call cerrorMsg(
+          if(nf .lt.  fc) then
+             call cerrorMsg(
      *      'No. of fields in primary data is too few', 1)
-          call cerrorMsg('data given is', 1)
-          call cerrorMsg(buf, 1)
-          stop 9999
+             call cerrorMsg('data given is', 1)
+             call cerrorMsg(buf, 1)
+             stop 9999
+          endif
        endif
 
        if(subcode .eq. 0 .and.  chg .eq. 0) then
@@ -321,6 +302,7 @@ c         nf many not be equal to fc. but must be nf >= fc
        endif
 
        call cmkptc( code, subcode, chg, aTrack.p)
+
        if(Inpxyz .eq. 1) then
           aTrack.pos.x = xin
           aTrack.pos.y = yin
@@ -339,10 +321,15 @@ c            for safety also set next
           DCInp.y = wy
           DCInp.z = wz
        endif
+
        aTrack.p.fm.p(4)  =  erg
-       if(Inperg .eq. 1) then
-          aTrack.p.fm.p(4) =  aTrack.p.fm.p(4) + aTrack.p.mass
+        ! >>>>>>>>>>>>>light
+       if( code > 0 .or. code == kchgPath ) then
+          if(Inperg .eq. 1) then
+             aTrack.p.fm.p(4) =  aTrack.p.fm.p(4) + aTrack.p.mass
+          endif
        endif
+        !<<<<<<<<<<<<<<<<<<
        icon = 0
        return
  100   icon = -1
@@ -351,6 +338,7 @@ c            for safety also set next
        subroutine  s1set(icon)
        implicit none
 #include  "ZsepManager.h"
+#include  "ZepTrackp.h"
 #include  "ZepPos.h"
 #include  "ZepDirec.h"
 #include  "Zep3Vec.h"
@@ -361,6 +349,7 @@ c            for safety also set next
 
       integer jcon
 
+
       if(nevc .ge. Nevent) then
          icon=1
       else
@@ -379,7 +368,6 @@ c                 init 1 event. (clear arrays etc)
          if(jcon .ne. 0) then
 c                    jcon=0-->user set the incident
 c                    set incident ptcl(s) by sepics
-
             call ssetip(icon)
          else
             icon = 0       ! this is needed 2004. May 2
@@ -394,7 +382,6 @@ c                  check time using stack area data
          if(icon .eq. 0) then
 c              time and incident are available  for the generation
 c              generate 1 event
-
             call epgen
 c              end of 1 event
             call epe1ev
@@ -436,8 +423,16 @@ c      *****************
       subroutine ssetip( icon )
       implicit none
 #include  "ZsepManager.h"
+#include  "ZepTrackp.h"
       integer icon  !  output.  0
+      integer eventn
       icon = 0
+      
+      if( Light == 22 ) then
+            ! must read primary and 1st col info.
+         call epLightIOread1stCol(eventn,icon)
+         if(icon /= 0 ) return
+      endif
 
       if( Inpmul .eq. 1) then
 c            multiple particles are incident; icon = 1--> end of a bunch
@@ -452,7 +447,7 @@ c            multiple particles are incident; icon = 1--> end of a bunch
 
       subroutine ssetip2( icon )
       implicit none
-
+#include  "Zcode.h"
 #include  "ZsepManager.h"
 #include  "ZepTrackp.h"
 #include  "ZepTrack.h"
@@ -469,16 +464,21 @@ c            multiple particles are incident; icon = 1--> end of a bunch
       record /epPos/ org, rxyz
       record /epDirec/ norm
       character*40 msg
-
+      real(8):: wl0 
 
 
 c             set incident ptcl
 c               fix energy (momentum yet)
       if(Inp1ry .eq. 0 ) then
          call sfixe(aTrack)
+         if( aTrack.p.code == klight ) then
+            call epLightE2wl(aTrack.p.fm.p(4), 1.d0, wl0, wl0)
+            aTrack.wl = wl0
+         endif
          icon = 0
       else
          call ssetip3(aTrack, icon)
+
          if(icon .ne. 0) return ! ************ EOF or end of 1 bunch of incidents
       endif
 
@@ -494,10 +494,7 @@ c               fix angle
          if(Inpdir .eq. 0) then
             call sfixa(aTrack)
          endif
-c/////////////
-c         write(0,*) ' pos=',PosInp.x, PosInp.y, PosInp.z
-c         write(0,*) ' ang=', DCInp.x, DCInp.y, DCInp.z
-c///////////
+
          if(Inpxyz .eq. 0.) then
             if(InputP .eq. 'fix') then
                aTrack.pos=PosInp
@@ -568,31 +565,27 @@ c      enddo
 c               fix others
 
       call sfixo(aTrack)
+
 c               set them
-      if(Inpdisk .eq. 0) then
-         call epputTrack(aTrack)
+c      if(Inpdisk .eq. 0) then
+       !>>>>>>>>>>>>>light
+      if( Inpcn > 0 ) then
+         ! if Cn is given, the coordinate is assumed to be local
+         ! so that eppush is ok
+         call eppush(aTrack)
       else
-         call epputTrk2(Inpdisk, aTrack)
+         call epputTrack(aTrack)
       endif
+       !<<<<<<<<<<<<<<
+c      else
+c         call epputTrk2(Inpdisk, aTrack)   ! not used now v9.00
+c      endif
       InciTrack = aTrack
       return
 c      *****************
        entry stimec(icon)
 c      *****************
-c         icon=0---> time available
-c             ^=1---> no //
-c              inquire current stack index
-       call epqstn(ns)
-c              inquire energy
-       sume=0.
-       do   i=1, ns
-          call epqstt(i, aTrack)
-          sume=sume+aTrack.p.fm.p(4)
-       enddo
-c             estimate time for energy:   BaseTime=time for 1gev
-           j=sume* BaseTime
-c             see if more cpu time available
-c           call timec(j, icon)
+c         icon=0---> time available;  currently no check
            icon = 0
        return
 c      ***************
@@ -616,7 +609,7 @@ c               end of all events
 cc          call scontw(IoCont)
 cc          call timepc(6)
 c              inquire # of events created in this run
-          call epqevn(nev)
+          call epqevn(nev)  
 c              print message
 c             get  current random no.
           call rnd1s(Ir1)
@@ -627,6 +620,7 @@ c             get  current random no.
           if(MsgLevel -1 .ge. 0) then
              call cerrorMsg(msg, 1)
           endif
+          call epcloseStackDisk  ! delete stack disk if used
        end
       subroutine sfixe(aTrack)
        implicit none
