@@ -23,9 +23,6 @@ c&      real*8 epsolverl
 c&      external epsolverl
 c     &&&&&&&&&&&&&&&&&&&&&&&&&&&
 
-      media.A = 0.
-      media.Z = 0.
-      media.Z2 = 0.
       nc = media.noOfElem
       sum = 0.
       do i = 1, nc
@@ -40,11 +37,25 @@ c
          media.No(i) = media.No(i)/sum
       enddo
       sum = 0.
+      media.A = 0.
+      media.Z = 0.
+      media.Z2 = 0.
+      media.Z1_3rd =0.
+      media.Z2_3rd =0.
+
       do i = 1, nc
          media.A = media.A + media.No(i)* media.elem(i).A
          media.Z = media.Z + media.No(i)* media.elem(i).Z
          media.Z2 = media.Z2 + media.No(i)* media.elem(i).Z**2
+         media.Z1_3rd =  media.Z1_3rd 
+     *          +  media.No(i)* media.elem(i).Z**(1./3.)
+         media.Z2_3rd =  media.Z2_3rd 
+     *          +  media.No(i)* media.elem(i).Z**(2./3.)
       enddo
+c        we simply deinfe  Z^(2/3) by <Z>^(2/3) (not <Z^2/3>
+c        also  Z^(1/3)
+c      media.Z1_3rd = media.Z**(1./3.)
+c      media.Z2_3rd = media.Z**(2./3.)
 
       do i = 1, nc
          media.w(i) = media.No(i)* media.elem(i).A/media.A
@@ -63,31 +74,34 @@ c         we use effective Z/A directly.
       do i =1, nc
          sum = sum + media.w(i)*media.elem(i).A
       enddo
-      media.Aeff = sum
+      media.Aeff = sum    ! not to be used alone
+
 c         get effective Z**2/A
       sum = 0
       do i = 1, nc
          sum = sum + media.w(i)*media.elem(i).Z**2/
      *         media.elem(i).A
       enddo
-      media.Z2byAeff = sum
+      media.Z2byAeff = sum   ! OK  = sum(niZi^2)/sum(niAi)
+
+      media.Z2eff = media.Z2byAeff * media.Aeff  ! not to be used alone
+      media.Zeff = sqrt(media.Z2eff)         ! not to be used  
 
-      media.Z2eff = media.Z2byAeff * media.Aeff
-      media.Zeff = sqrt(media.Z2eff)
 c  &&&&&&&&&&&&&&&
 c      write(*,*) ' meida.No', ( media.No(i), i=1,nc )
 c      write(*,*) ' meida.w ', ( media.w(i), i=1,nc )
 c      write(*,*) ' A,Z, Aeff, Zeff=', media.A, media.Z, media.Aeff,
 c     *   media.Zeff
 c &&&&&&&&&&&&&&&&
-      media.LogZ = log(media.Zeff)
-      media.Zeff3 = media.Zeff**(1.d0/3.d0)
+      media.LogZ = log(media.Zeff)  ! not used anywhere
+      media.Zeff3 = media.Zeff**(1.d0/3.d0)   ! not to be used
 
       sum = 0.
       do i = 1,nc
          sum  =  sum +
      *             media.w(i)*media.elem(i).Z/media.elem(i).A
       enddo
+c         =  sum(niZi)/sum(niAi)  OK
       if( media.format .eq. 2 .and. media.ZbyAeff .ne. -100.) then
          if(abs(sum-media.ZbyAeff)/media.ZbyAeff .gt. 0.01 ) then
             write(0,*) "weighted <Z/A> inp=",media.ZbyAeff,
@@ -103,7 +117,7 @@ c            for photo electric effect
          sum = sum + 
      *        media.w(i)*media.elem(i).Z**5/media.elem(i).A
       enddo
-      media.Z5byAeff = sum
+      media.Z5byAeff = sum   ! ok
 
 
 c         compute radiation length X0g(g/cm2) and  X0 (cm)
@@ -122,7 +136,9 @@ c         <Z/A> must be used explicitly for crosssections with Z
 c
 c           this may be used when we approximate
 c      a compound or molecule as one element
-      media.mbtoPgrm2 = 1.d-27*N0/media.Aeff
+c        wrong: ! not used anywhere
+c
+      media.mbtoPgrm2 = 1.d-27*N0/media.Aeff   
       media.mbtoPcm2 =  media.mbtoPgrm2 * media.rho
       media.mbtoPX02 =  media.mbtoPgrm2 * media.X0g
 c  &&&&&&&&&&&&&&&&&&&&&&&&
@@ -136,7 +152,8 @@ c&       write(*,*) ' newz=',newz
 c  &&&&&&&&&&&&&&&&&&&&&&&
 c       ------------LPM -----------;
 c        these are not atom base. but media base.
-      media.s1 = (media.Zeff**(1.d0/3.d0)/183.0d0)**2
+c      media.s1 = (media.Zeff**(1.d0/3.d0)/183.0d0)**2
+      media.s1 = (media.Z1_3rd/183.0d0)**2
       media.logs1 = log(media.s1)
 c       -----------complete screening cross-sec. coeff.
 
